@@ -13,28 +13,36 @@ func galMulNEON(c uint64, in, out []byte)
 //go:noescape
 func galMulXorNEON(c uint64, in, out []byte)
 
-func galMulSlice(c byte, in, out []byte, ssse3, avx2 bool) {
+func galMulSlice(c byte, in, out []byte, o *options) {
+	if c == 1 {
+		copy(out, in)
+		return
+	}
 	var done int
 	galMulNEON(uint64(c), in, out)
 	done = (len(in) >> 5) << 5
 
 	remain := len(in) - done
 	if remain > 0 {
-		mt := mulTable[c]
+		mt := mulTable[c][:256]
 		for i := done; i < len(in); i++ {
 			out[i] = mt[in[i]]
 		}
 	}
 }
 
-func galMulSliceXor(c byte, in, out []byte, ssse3, avx2 bool) {
+func galMulSliceXor(c byte, in, out []byte, o *options) {
+	if c == 1 {
+		sliceXor(in, out, o)
+		return
+	}
 	var done int
 	galMulXorNEON(uint64(c), in, out)
 	done = (len(in) >> 5) << 5
 
 	remain := len(in) - done
 	if remain > 0 {
-		mt := mulTable[c]
+		mt := mulTable[c][:256]
 		for i := done; i < len(in); i++ {
 			out[i] ^= mt[in[i]]
 		}
@@ -42,7 +50,7 @@ func galMulSliceXor(c byte, in, out []byte, ssse3, avx2 bool) {
 }
 
 // slice galois add
-func sliceXor(in, out []byte, sse2 bool) {
+func sliceXor(in, out []byte, o *options) {
 	for n, input := range in {
 		out[n] ^= input
 	}
